@@ -219,3 +219,151 @@ pub(crate) fn assert_session_ne(name: &str) {
     };
     process::exit(1);
 }
+
+/// Create a new random name generator
+///
+/// Used to provide a memorable handle for a session when users don't specify a session name when the session is
+/// created.
+///
+/// Uses the list of adjectives and nouns defined below, with the intention of avoiding unfortunate
+/// and offensive combinations. Care should be taken when adding or removing to either list due to the birthday paradox/
+/// hash collisions, e.g. with 4096 unique names, the likelihood of a collision in 10 session names is 1%.
+pub(crate) fn get_name_generator() -> impl Iterator<Item = String> {
+    names::Generator::new(&ADJECTIVES, &NOUNS, names::Name::Plain)
+}
+
+const ADJECTIVES: &[&'static str] = &[
+    "adamant",
+    "adept",
+    "adventurous",
+    "arcadian",
+    "auspicious",
+    "awesome",
+    "blossoming",
+    "brave",
+    "charming",
+    "chatty",
+    "circular",
+    "considerate",
+    "cubic",
+    "curious",
+    "delighted",
+    "didactic",
+    "diligent",
+    "effulgent",
+    "erudite",
+    "excellent",
+    "exquisite",
+    "fabulous",
+    "fascinating",
+    "friendly",
+    "glowing",
+    "gracious",
+    "gregarious",
+    "hopeful",
+    "implacable",
+    "inventive",
+    "joyous",
+    "judicious",
+    "jumping",
+    "kind",
+    "likable",
+    "loyal",
+    "lucky",
+    "marvellous",
+    "mellifluous",
+    "nautical",
+    "oblong",
+    "outstanding",
+    "polished",
+    "polite",
+    "profound",
+    "quadratic",
+    "quiet",
+    "rectangular",
+    "remarkable",
+    "rusty",
+    "sensible",
+    "sincere",
+    "sparkling",
+    "splendid",
+    "stellar",
+    "tenacious",
+    "tremendous",
+    "triangular",
+    "undulating",
+    "unflappable",
+    "unique",
+    "verdant",
+    "vitreous",
+    "wise",
+    "zippy",
+];
+
+const NOUNS: &[&'static str] = &[
+    "aardvark",
+    "accordion",
+    "apple",
+    "apricot",
+    "bee",
+    "brachiosaur",
+    "cactus",
+    "capsicum",
+    "clarinet",
+    "cowbell",
+    "crab",
+    "cuckoo",
+    "cymbal",
+    "diplodocus",
+    "donkey",
+    "drum",
+    "duck",
+    "echidna",
+    "elephant",
+    "foxglove",
+    "galaxy",
+    "glockenspiel",
+    "goose",
+    "hill",
+    "horse",
+    "iguanadon",
+    "jellyfish",
+    "kangaroo",
+    "lake",
+    "lemon",
+    "lemur",
+    "magpie",
+    "megalodon",
+    "mountain",
+    "mouse",
+    "muskrat",
+    "newt",
+    "oboe",
+    "ocelot",
+    "orange",
+    "panda",
+    "peach",
+    "pepper",
+    "petunia",
+    "pheasant",
+    "piano",
+    "pigeon",
+    "platypus",
+    "quasar",
+    "rhinoceros",
+    "river",
+    "rustacean",
+    "salamander",
+    "sitar",
+    "stegosaurus",
+    "tambourine",
+    "tiger",
+    "tomato",
+    "triceratops",
+    "ukulele",
+    "viola",
+    "weasel",
+    "xylophone",
+    "yak",
+    "zebra",
+];
