@@ -68,7 +68,6 @@ pub struct TiledPanes {
     draw_pane_frames: bool,
     panes_to_hide: HashSet<PaneId>,
     fullscreen_is_active: bool,
-    os_api: Box<dyn ServerOsApi>,
     senders: ThreadSenders,
     window_title: Option<String>,
     client_id_to_boundaries: HashMap<ClientId, Boundaries>,
@@ -105,7 +104,6 @@ impl TiledPanes {
             draw_pane_frames,
             panes_to_hide: HashSet::new(),
             fullscreen_is_active: false,
-            os_api,
             senders,
             window_title: None,
             client_id_to_boundaries: HashMap::new(),
@@ -182,9 +180,13 @@ impl TiledPanes {
         let has_room_for_new_pane = pane_grid
             .find_room_for_new_pane(cursor_height_width_ratio)
             .is_some();
-        has_room_for_new_pane || pane_grid.has_room_for_new_stacked_pane()
+        has_room_for_new_pane || pane_grid.has_room_for_new_stacked_pane() || self.panes.is_empty()
     }
     fn add_pane(&mut self, pane_id: PaneId, mut pane: Box<dyn Pane>, should_relayout: bool) {
+        if self.panes.is_empty() {
+            self.panes.insert(pane_id, pane);
+            return;
+        }
         let cursor_height_width_ratio = self.cursor_height_width_ratio();
         let mut pane_grid = TiledPaneGrid::new(
             &mut self.panes,
@@ -250,6 +252,19 @@ impl TiledPanes {
             })
             .collect()
     }
+    pub fn non_selectable_pane_geoms_inside_viewport(&self) -> Vec<Viewport> {
+        self.panes
+            .values()
+            .filter_map(|p| {
+                let geom = p.position_and_size();
+                if !p.selectable() && is_inside_viewport(&self.viewport.borrow(), p) {
+                    Some(geom.into())
+                } else {
+                    None
+                }
+            })
+            .collect()
+    }
     pub fn first_selectable_pane_id(&self) -> Option<PaneId> {
         self.panes
             .iter()
@@ -508,6 +523,11 @@ impl TiledPanes {
         }
     }
     pub fn focus_pane(&mut self, pane_id: PaneId, client_id: ClientId) {
+        if self.panes_to_hide.contains(&pane_id) {
+            // this means there is a fullscreen pane that is not the current pane, let's unset it
+            // before changing focus
+            self.unset_fullscreen();
+        }
         if self
             .panes
             .get(&pane_id)
@@ -518,7 +538,6 @@ impl TiledPanes {
                 .expand_pane(&pane_id);
             self.reapply_pane_frames();
         }
-
         self.active_panes
             .insert(client_id, pane_id, &mut self.panes);
         if self.session_is_mirrored {
@@ -569,8 +588,9 @@ impl TiledPanes {
         }
     }
     pub fn focus_pane_if_client_not_focused(&mut self, pane_id: PaneId, client_id: ClientId) {
-        if self.active_panes.get(&client_id).is_none() {
-            self.focus_pane(pane_id, client_id)
+        match self.active_panes.get(&client_id) {
+            Some(already_focused_pane_id) => self.focus_pane(*already_focused_pane_id, client_id),
+            None => self.focus_pane(pane_id, client_id),
         }
     }
     pub fn clear_active_panes(&mut self) {
@@ -587,8 +607,6 @@ impl TiledPanes {
     pub fn focused_pane_id(&self, client_id: ClientId) -> Option<PaneId> {
         self.active_panes.get(&client_id).copied()
     }
-    // FIXME: Really not a fan of allowing this... Someone with more energy
-    // than me should clean this up someday...
     #[allow(clippy::borrowed_box)]
     pub fn get_pane(&self, pane_id: PaneId) -> Option<&Box<dyn Pane>> {
         self.panes.get(&pane_id)
@@ -623,7 +641,7 @@ impl TiledPanes {
             { self.connected_clients.borrow().iter().copied().collect() };
         let multiple_users_exist_in_session = { self.connected_clients_in_app.borrow().len() > 1 };
         let mut client_id_to_boundaries: HashMap<ClientId, Boundaries> = HashMap::new();
-        let active_panes = if self.session_is_mirrored || floating_panes_are_visible {
+        let active_panes = if floating_panes_are_visible {
             HashMap::new()
         } else {
             self.active_panes
@@ -735,6 +753,29 @@ impl TiledPanes {
     pub fn get_panes(&self) -> impl Iterator<Item = (&PaneId, &Box<dyn Pane>)> {
         self.panes.iter()
     }
+    pub fn set_geom_for_pane_with_run(
+        &mut self,
+        run: Option<Run>,
+        geom: PaneGeom,
+        borderless: bool,
+    ) {
+        match self
+            .panes
+            .iter_mut()
+            .find(|(_, p)| p.invoked_with() == &run)
+        {
+            Some((_, pane)) => {
+                pane.set_geom(geom);
+                pane.set_borderless(borderless);
+                if self.draw_pane_frames {
+                    pane.set_content_offset(Offset::frame(1));
+                }
+            },
+            None => {
+                log::error!("Failed to find pane with run: {:?}", run);
+            },
+        }
+    }
     pub fn resize(&mut self, new_screen_size: Size) {
         // this is blocked out to appease the borrow checker
         {
@@ -1501,11 +1542,11 @@ impl TiledPanes {
             self.set_pane_frames(self.draw_pane_frames); // recalculate pane frames and update size
             closed_pane
         } else {
-            self.panes.remove(&pane_id);
+            let closed_pane = self.panes.remove(&pane_id);
             // this is a bit of a roundabout way to say: this is the last pane and so the tab
             // should be destroyed
             self.active_panes.clear(&mut self.panes);
-            None
+            closed_pane
         }
     }
     pub fn hold_pane(
